;; -*- no-byte-compile: t; lexical-binding: nil -*-
(define-package "darkman" "20260414.1055"
  "Seamless integration with Darkman."
  '((emacs "28.1"))
  :url "https://darkman.grtcdr.tn"
  :commit "ee99dfcdc1f48330d0fbb565f8db7664eee34643"
  :revdesc "ee99dfcdc1f4"
  :keywords '("convenience")
  :authors '(("Taha Aziz Ben Ali" . "ba.tahaaziz@gmail.com"))
  :maintainers '(("Taha Aziz Ben Ali" . "ba.tahaaziz@gmail.com")))
